@@ -1,4 +1,6 @@
 
+pred_horizon <- c(1000, 2500)
+
 test_preds_surv <- function(pred_type){
 
  n_train <- nrow(pbc_train)
@@ -10,6 +12,7 @@ test_preds_surv <- function(pred_type){
   surv = length(pred_horizon),
   chf  = length(pred_horizon),
   mort = 1,
+  time = 1,
   leaf = n_tree_test
  )
 
@@ -30,7 +33,7 @@ test_preds_surv <- function(pred_type){
              oobag_pred_horizon = pred_horizon,
              tree_seeds = seeds_standard)
 
- if(pred_type %in% c("mort", "leaf")) pred_horizon <- NULL
+ if(pred_type %in% c("mort", "leaf", "time")) pred_horizon <- NULL
 
  prd_agg <- predict(fit,
                     new_data = pbc_test,
@@ -41,6 +44,7 @@ test_preds_surv <- function(pred_type){
  prd_raw <- predict(fit,
                     new_data = pbc_test,
                     pred_aggregate = FALSE,
+                    pred_simplify = TRUE,
                     pred_type = pred_type,
                     pred_horizon = pred_horizon,
                     n_thread = 1)
@@ -70,7 +74,7 @@ test_preds_surv <- function(pred_type){
 
  }
 
- if(pred_type == 'mort'){
+ if(pred_type %in% c('mort', 'time')){
 
   test_that(
    desc = "predictions are accurate",
@@ -81,9 +85,13 @@ test_preds_surv <- function(pred_type){
      data = pbc_test
     )
 
-    mort_cstat <- 1 - surv_concord$concordance
+    cstat <- surv_concord$concordance
 
-    expect_true(mort_cstat > 0.60)
+    if(pred_type == 'mort'){
+     cstat <- 1 - cstat
+    }
+
+    expect_true(cstat > 0.60)
 
    }
   )
@@ -118,6 +126,7 @@ test_preds_surv <- function(pred_type){
             new_data = pbc_test,
             pred_aggregate = FALSE,
             pred_type = pred_type,
+            pred_simplify = TRUE,
             pred_horizon = pred_horizon,
             n_thread = 3)
    )
@@ -129,10 +138,411 @@ test_preds_surv <- function(pred_type){
 
 }
 
-pred_horizon <- c(1000, 2500)
+test_preds_clsf_multi <- function(pred_type){
+
+ n_train <- nrow(penguins_train)
+ n_test <- nrow(penguins_test)
+
+ pred_ncols_expect_agg <- switch(
+  pred_type,
+  prob = 3,
+  class = 1,
+  leaf = n_tree_test
+ )
+
+ dim_expect_agg <- list(
+  oob = c(n_train, pred_ncols_expect_agg),
+  new = c(n_test, pred_ncols_expect_agg)
+ )
+
+ dim_expect_raw <- c(n_test, n_tree_test)
+
+ fit <- orsf(formula = species ~ .,
+             data = penguins_train,
+             oobag_pred_type = pred_type,
+             n_tree = n_tree_test,
+             tree_seeds = seeds_standard)
+
+ prd_agg <- predict(fit,
+                    new_data = penguins_test,
+                    pred_type = pred_type,
+                    n_thread = 1)
+
+ if(pred_type == 'prob'){
+
+  expect_error(predict(fit,
+                       new_data = penguins_test,
+                       pred_aggregate = FALSE,
+                       pred_type = pred_type,
+                       n_thread = 1),
+               regexp = 'unaggregated')
+
+  prd_raw <- NULL
+
+ } else {
+
+  prd_raw <- predict(fit,
+                     new_data = penguins_test,
+                     pred_aggregate = FALSE,
+                     pred_type = pred_type,
+                     n_thread = 1)
+
+  test_that(
+   'No missing, nan, or infinite values in unaggregated predictions',
+   code = {
+    expect_false(any(is.na(prd_raw)))
+    expect_false(any(is.nan(prd_raw)))
+    expect_false(any(is.infinite(prd_raw)))
+   }
+  )
+
+  test_that(
+   "unaggregated prediction dimensions match expectations",
+   code = {
+    expect_equal(dim_expect_raw, dim(prd_raw))
+   }
+  )
+
+ }
+
+ test_that(
+  'No missing, nan, or infinite values in aggregated prediction output',
+  code = {
+   expect_false(any(is.na(prd_agg)))
+   expect_false(any(is.nan(prd_agg)))
+   expect_false(any(is.infinite(prd_agg)))
+  }
+ )
+
+
+ if(pred_type == "prob"){
+
+  test_that(
+   desc = "probability predictions are bounded",
+   code = {
+    expect_true(all(prd_agg <= 1))
+    expect_true(all(prd_agg >= 0))
+   }
+  )
+
+  test_that(
+   desc = "probability predictions sum to 1",
+   code = {
+    expect_equal(apply(prd_agg, 1, sum), rep(1, nrow(prd_agg)),
+                 tolerance = .Machine$double.eps)
+   }
+  )
+
+ }
+
+ if(pred_type != 'leaf'){
+  test_that(
+   desc = "predictions are accurate",
+   code = {
+
+    if(pred_type == 'prob'){
+     prd_class <- apply(prd_agg, 1, which.max)
+    } else if (pred_type == 'class') {
+     prd_class <- as.numeric(prd_agg)
+    }
+
+    accuracy <- mean(prd_class == as.numeric(penguins_test$species))
+
+    expect_true(accuracy > 0.90)
+
+   }
+  )
+ }
+
+ test_that(
+  desc = paste(pred_type, "aggregated prediction dimensions match expectations"),
+  code = {
+   expect_equal(dim_expect_agg$oob, dim(fit$pred_oobag))
+   expect_equal(dim_expect_agg$new, dim(prd_agg))
+  }
+ )
+
+ test_that(
+  desc = paste('thread stability for predictions of type', pred_type),
+  code = {
+   expect_equal(
+    prd_agg,
+    predict(fit,
+            new_data = penguins_test,
+            pred_type = pred_type,
+            n_thread = 0)
+   )
+  }
+ )
+
+ if(pred_type == 'class'){
+  expect_equal(
+   prd_raw,
+   predict(fit,
+           new_data = penguins_test,
+           pred_aggregate = FALSE,
+           pred_type = pred_type,
+           n_thread = 0)
+  )
+ }
+
+ list(fit = fit, prd_agg = prd_agg, prd_raw = prd_raw)
+
+}
+
+test_preds_clsf_bnry <- function(pred_type){
+
+ n_train <- nrow(penguins_binary_train)
+ n_test <- nrow(penguins_binary_test)
+
+ pred_ncols_expect_agg <- switch(
+  pred_type,
+  prob = 2,
+  class = 1,
+  leaf = n_tree_test
+ )
+
+ dim_expect_agg <- list(
+  oob = c(n_train, pred_ncols_expect_agg),
+  new = c(n_test, pred_ncols_expect_agg)
+ )
+
+ dim_expect_raw <- c(n_test, n_tree_test)
+
+ fit <- orsf(formula = species ~ .,
+             data = penguins_binary_train,
+             oobag_pred_type = pred_type,
+             n_tree = n_tree_test,
+             tree_seeds = seeds_standard)
+
+ prd_agg <- predict(fit,
+                    new_data = penguins_binary_test,
+                    pred_type = pred_type,
+                    n_thread = 1)
+
+ prd_raw <- predict(fit,
+                    new_data = penguins_binary_test,
+                    pred_aggregate = FALSE,
+                    pred_type = pred_type,
+                    n_thread = 1)
+
+  test_that(
+   'No missing, nan, or infinite values in unaggregated predictions',
+   code = {
+    expect_false(any(is.na(prd_raw)))
+    expect_false(any(is.nan(prd_raw)))
+    expect_false(any(is.infinite(prd_raw)))
+   }
+  )
+
+  test_that(
+   "unaggregated prediction dimensions match expectations",
+   code = {
+    expect_equal(dim_expect_raw, dim(prd_raw))
+   }
+  )
+
+ test_that(
+  'No missing, nan, or infinite values in aggregated prediction output',
+  code = {
+   expect_false(any(is.na(prd_agg)))
+   expect_false(any(is.nan(prd_agg)))
+   expect_false(any(is.infinite(prd_agg)))
+  }
+ )
+
+
+ if(pred_type == "prob"){
+
+  test_that(
+   desc = "probability predictions are bounded",
+   code = {
+    expect_true(all(prd_agg <= 1))
+    expect_true(all(prd_agg >= 0))
+   }
+  )
+
+  test_that(
+   desc = "probability predictions sum to 1",
+   code = {
+    expect_equal(apply(prd_agg, 1, sum), rep(1, nrow(prd_agg)),
+                 tolerance = .Machine$double.eps)
+   }
+  )
+
+ }
+
+ if(pred_type != 'leaf'){
+  test_that(
+   desc = "predictions are accurate",
+   code = {
+
+    if(pred_type == 'prob'){
+     prd_class <- apply(prd_agg, 1, which.max)
+    } else if (pred_type == 'class') {
+     prd_class <- as.numeric(prd_agg)
+    }
+
+    accuracy <- mean(prd_class == as.numeric(penguins_binary_test$species))
+
+    expect_true(accuracy > 0.90)
+
+   }
+  )
+ }
+
+ test_that(
+  desc = paste(pred_type, "aggregated prediction dimensions match expectations"),
+  code = {
+   expect_equal(dim_expect_agg$oob, dim(fit$pred_oobag))
+   expect_equal(dim_expect_agg$new, dim(prd_agg))
+  }
+ )
+
+ test_that(
+  desc = paste('thread stability for predictions of type', pred_type),
+  code = {
+   expect_equal(
+    prd_agg,
+    predict(fit,
+            new_data = penguins_binary_test,
+            pred_type = pred_type,
+            n_thread = 0)
+   )
+  }
+ )
+
+ if(pred_type == 'class'){
+  expect_equal(
+   prd_raw,
+   predict(fit,
+           new_data = penguins_binary_test,
+           pred_aggregate = FALSE,
+           pred_type = pred_type,
+           n_thread = 0)
+  )
+ }
+
+ list(fit = fit, prd_agg = prd_agg, prd_raw = prd_raw)
+
+}
+
+test_preds_regr <- function(pred_type){
+
+ n_train <- nrow(penguins_train)
+ n_test <- nrow(penguins_test)
+
+ pred_ncols_expect_agg <- switch(
+  pred_type,
+  mean = 1,
+  leaf = n_tree_test
+ )
+
+ dim_expect_agg <- list(
+  oob = c(n_train, pred_ncols_expect_agg),
+  new = c(n_test, pred_ncols_expect_agg)
+ )
+
+ dim_expect_raw <- c(n_test, n_tree_test)
+
+ fit <- orsf(formula = bill_depth_mm ~ .,
+             data = penguins_train,
+             oobag_pred_type = pred_type,
+             n_tree = n_tree_test,
+             tree_seeds = seeds_standard)
+
+ prd_agg <- predict(fit,
+                    new_data = penguins_test,
+                    pred_type = pred_type,
+                    n_thread = 1)
+
+ prd_raw <- predict(fit,
+                    new_data = penguins_test,
+                    pred_aggregate = FALSE,
+                    pred_type = pred_type,
+                    n_thread = 1)
+
+ test_that(
+  'No missing, nan, or infinite values in unaggregated predictions',
+  code = {
+   expect_false(any(is.na(prd_raw)))
+   expect_false(any(is.nan(prd_raw)))
+   expect_false(any(is.infinite(prd_raw)))
+  }
+ )
+
+ test_that(
+  "unaggregated prediction dimensions match expectations",
+  code = {
+   expect_equal(dim_expect_raw, dim(prd_raw))
+  }
+ )
+
+ test_that(
+  'No missing, nan, or infinite values in aggregated prediction output',
+  code = {
+   expect_false(any(is.na(prd_agg)))
+   expect_false(any(is.nan(prd_agg)))
+   expect_false(any(is.infinite(prd_agg)))
+  }
+ )
+
+ if(pred_type != 'leaf'){
+  test_that(
+   desc = "predictions are accurate",
+   code = {
+
+    y_mean <- mean(penguins_train$bill_depth_mm)
+    ssq <- mean((y_mean - penguins_test$bill_depth_mm)^2)
+    mse <- mean((prd_agg - penguins_test$bill_depth_mm)^2)
+    rsq <- 1 - (mse / ssq)
+
+    expect_true(rsq > 0.70)
+
+   }
+  )
+ }
+
+ test_that(
+  desc = paste(pred_type, "aggregated prediction dimensions match expectations"),
+  code = {
+   expect_equal(dim_expect_agg$oob, dim(fit$pred_oobag))
+   expect_equal(dim_expect_agg$new, dim(prd_agg))
+  }
+ )
+
+ test_that(
+  desc = paste('thread stability for predictions of type', pred_type),
+  code = {
+   expect_equal(
+    prd_agg,
+    predict(fit,
+            new_data = penguins_test,
+            pred_type = pred_type,
+            n_thread = 0)
+   )
+   expect_equal(
+    prd_raw,
+    predict(fit,
+            new_data = penguins_test,
+            pred_aggregate = FALSE,
+            pred_type = pred_type,
+            n_thread = 0)
+   )
+  }
+ )
+
+ list(fit = fit, prd_agg = prd_agg, prd_raw = prd_raw)
+
+}
+
 
 pred_objects_surv <- lapply(pred_types_surv, test_preds_surv)
 
+pred_objects_clsf_multi <- lapply(pred_types_clsf, test_preds_clsf_multi)
+pred_objects_clsf_bnry <- lapply(pred_types_clsf, test_preds_clsf_bnry)
+pred_objects_regr <- lapply(pred_types_regr, test_preds_regr)
+
 test_that(
  desc = "prediction at time 0 is correct",
  code = {
@@ -151,35 +561,35 @@ test_that(
  }
 )
 
-# test_that(
-#  desc = "leaf predictions aggregate same as raw",
-#  code = {
-#   expect_equal(pred_objects_surv$leaf$prd_raw,
-#                pred_objects_surv$leaf$prd_agg)
-#  }
-# )
+test_that(
+ desc = "leaf predictions aggregate same as raw",
+ code = {
+  expect_equal(pred_objects_surv$leaf$prd_raw,
+               pred_objects_surv$leaf$prd_agg)
+ }
+)
 
-# test_that(
-#  desc = "unaggregated predictions can reproduce aggregated ones",
-#  code = {
-#
-#   for(i in c("surv", "risk", "chf")){
-#    for(j in seq_along(pred_horizon)){
-#     expect_equal(
-#      pred_objects_surv[[i]]$prd_agg[, j],
-#      apply(pred_objects_surv[[i]]$prd_raw[, , j], 1, mean),
-#      tolerance = 1e-9
-#     )
-#    }
-#   }
-#
-#   expect_equal(
-#    pred_objects_surv$mort$prd_agg,
-#    matrix(apply(pred_objects_surv$mort$prd_raw, 1, mean), ncol = 1)
-#   )
-#
-#  }
-# )
+test_that(
+ desc = "unaggregated predictions can reproduce aggregated ones",
+ code = {
+
+  for(i in c("surv", "risk", "chf")){
+   for(j in seq_along(pred_horizon)){
+    expect_equal(
+     pred_objects_surv[[i]]$prd_agg[, j],
+     apply(pred_objects_surv[[i]]$prd_raw[, , j], 1, mean),
+     tolerance = 1e-9
+    )
+   }
+  }
+
+  expect_equal(
+   pred_objects_surv$mort$prd_agg,
+   matrix(apply(pred_objects_surv$mort$prd_raw, 1, mean), ncol = 1)
+  )
+
+ }
+)
 
 test_that(
  desc = "same predictions from the forest regardless of oob type",
@@ -524,7 +934,7 @@ test_that(
            pred_horizon = 100000,
            boundary_checks = F),
    predict(fit, pbc_test,
-           pred_horizon = get_max_time(fit))
+           pred_horizon = max(pbc_train$time))
   )
  }
 )
@@ -557,23 +967,6 @@ test_that(
 )
 
 
-# test_that(
-#  desc = 'missing units are detected',
-#  code = {
-#
-#   suppressMessages(library(units))
-#   pbc_units <- pbc_orsf
-#   units(pbc_units$age) <- 'years'
-#
-#   fit <- orsf(formula = time + status  ~ . - id,
-#               data = pbc_units,
-#               n_tree = n_tree_test)
-#
-#   expect_error(predict(fit, new_data = pbc_orsf, pred_horizon = 1000),
-#                'unit attributes')
-#
-#  }
-# )
 
 test_that(
  desc = 'predictions dont require cols in same order as training data',
@@ -594,59 +987,6 @@ test_that(
 )
 
 
-# test_that(
-#  'units are vetted in testing data',
-#  code = {
-#
-#   suppressMessages(library(units))
-#   pbc_units_trn <- pbc_train
-#   pbc_units_tst <- pbc_test
-#
-#
-#   units(pbc_units_trn$time) <- 'days'
-#   units(pbc_units_trn$age) <- 'years'
-#   units(pbc_units_trn$bili) <- 'mg/dl'
-#
-#   fit_units = orsf(formula = time + status  ~ . - id,
-#                    data = pbc_units_trn,
-#                    n_tree = n_tree_test,
-#                    oobag_pred_horizon = c(1000, 2500),
-#                    tree_seeds = seeds_standard)
-#
-#   expect_error(
-#    predict(fit_units, new_data = pbc_units_tst, pred_horizon = 1000),
-#    regexp = 'time, age, and bili'
-#   )
-#
-#   units(pbc_units_tst$time) <- 'years'
-#   units(pbc_units_tst$age) <- 'years'
-#   units(pbc_units_tst$bili) <- 'mg/dl'
-#
-#   expect_error(
-#    predict(fit_units, new_data = pbc_units_tst, pred_horizon = 1000),
-#    regexp = 'time has unit d in the training data'
-#   )
-#
-#   units(pbc_units_tst$time) <- 'days'
-#   units(pbc_units_tst$age) <- 'years'
-#   units(pbc_units_tst$bili) <- 'mg/dl'
-#
-#   expect_equal_leaf_summary(fit_units, pred_objects_surv$surv$fit)
-#   expect_equal_oobag_eval(fit_units, pred_objects_surv$surv$fit)
-#
-#   units(pbc_units_tst$time) <- 'days'
-#   units(pbc_units_tst$age) <- 'years'
-#   units(pbc_units_tst$bili) <- 'mg/l'
-#
-#   expect_error(
-#    predict(fit_units, new_data = pbc_units_tst, pred_horizon = 1000),
-#    regexp = 'bili has unit mg/dl in the training data'
-#   )
-#
-#  }
-#
-# )
-
 # Tests for passing missing data ----
 
 na_index_age <- c(1, 4, 8)
@@ -787,11 +1127,11 @@ new_data_all_miss <- new_data_miss
 new_data_all_miss$age <- NA_real_
 
 test_that(
- desc = "can't give orsf nothing but missing data",
+ desc = "no blank columns allowed",
  code = {
   expect_error(
    predict(fit, new_data = new_data_all_miss, na_action = 'pass'),
-   regexp = 'complete data'
+   regexp = 'age has no observed values'
   )
  }
 )
