@@ -8,18 +8,18 @@ test_that(
                                      'pbc_status_12',
                                      'pbc_scaled')])){
 
-   for(j in seq_along(controls)){
+   for(j in seq_along(controls_surv)){
 
     fit_untrained <- orsf(data_list_pbc[[i]],
                           formula = time + status ~ . - id,
                           n_tree = n_tree_test,
-                          control = controls[[j]],
+                          control = controls_surv[[j]],
                           tree_seed = seeds_standard,
                           no_fit = TRUE)
 
     expect_true(is_empty(fit_untrained$eval_oobag$stat_values))
 
-    expect_equal( attr(fit_untrained, 'trained'), FALSE )
+    expect_false(fit_untrained$trained)
 
     fit_trained <- orsf_train(fit_untrained)
 
